searchNodes=[{"ref":"SSH.html","title":"SSH","type":"module","doc":"SSH streams and SSH and basic SCP functionality The librarian SSH module provides SSH streams (see stream/3) and three protocols over the SSH stream: run/3, which runs a command on the remote SSH host. fetch/3, which uses the SCP protocol to obtain a remote file, send/4, which uses the SCP protocol to send a file to the remote host. Note that not all SSH hosts (for example, embedded shells), implement an SCP command, so you may not necessarily be able to perform SCP over your SSH stream. Using SSH The principles of this library are simple. You will first want to create an SSH connection using the connect/2 function. There you will provide credentials (or let the system figure out the default credentials). The returned conn term can then be passed to the multiple utilities. {:ok, conn} = SSH.connect(&quot;some.other.server&quot;) SSH.run!(conn, &quot;echo hello ssh&quot;) # ==&gt; &quot;hello ssh&quot; Bang vs non-bang functions As a general rule, if you expect to run a single or series of tasks with transient (or no) supervision, for example in a worker task or elixir script you should use the bang function and let the task fail, designing your supervision accordingly. This will also potentially let you be lazy about system resources such as SSH connections. If you expect your SSH task to run as a part of a long-running process (for example, checking in on a host and retrieving data), you should use the error tuple forms and also be careful about closing your ssh connections after use. Check the connection labels documentation for a strategy to organize your code around this neatly. Mocking There&#39;s a good chance you&#39;ll want to mock your SSH commands and responses. The SSH.Api behaviour module is provided for that purpose. Logging The SSH and related modules interface with Elixir (and Erlang&#39;s) logging facility. The default metadata tagged on the message is ssh: true; if you would like to set it otherwise you can set the :librarian, :ssh_metadata application environment variable. Customization If you would like to write your own SSH stream handlers that plug in to the SSH stream and provide either rudimentary interactivity or early stream token processing, you may want to consider implementing a module following the SSH.ModuleApi behaviour, and initiating your stream as desired. Limitations This library has largely been tested against Linux SSH clients. Not all SSH schemes are amenable to stream processing. In those cases you should implement an ssh client gen_server using erlang&#39;s ssh_client, though support for this in elixir is planned in the near-term."},{"ref":"SSH.html#close/1","title":"SSH.close/1","type":"function","doc":"closes the ssh connection. Typically you will pass the connection reference to this function. If your connection is contained to its own transient task process, you may not need to call this function as the ssh client library will detect that the process has ended and clean up after you. In some cases, you may want to be able to close a connection out-of-band. In this case, you may label your connection and use the label to perform the close operation. See labels"},{"ref":"SSH.html#connect/2","title":"SSH.connect/2","type":"function","doc":"initiates an ssh connection with a remote server. options: :use_ssh_config see SSH.Config, defaults to false. :global_config_path see SSH.Config. :user_config_path see SSH.Config. :user username to log in as. :port port to use to ssh, defaults to 22. :label see labels and other SSH options. Some conversions between ssh options and SSH.connect options: ssh commandline optionSSH library option -o StrictHostKeyChecking=nosilently_accept_hosts: true -qquiet_mode: true -o ConnectTimeout=timeconnect_timeout: time_in_ms -i pemfileidentity: file also consult documentation on client options in the erlang docs labels: You can label your ssh connection to provide a side-channel for correctly closing the connection pid. This is most useful in the context of with/1 blocks. As an example, the following code works: def run_ssh_tasks do with {:ok, conn} &lt;- SSH.connect(&quot;some_host&quot;, label: :this_task), {:ok, _result1, 0} &lt;- SSH.run(conn, &quot;some_command&quot;), {:ok, result2, 0} &lt;- SSH.run(conn, &quot;some other command&quot;) do {:ok, result1} end after SSH.close(:this_task) end Some important points: If you are wrangling multiple SSH sessions, please use unique connection labels. The ssh connection label is stored in the process dictionary, so the label will not be valid across process boundaries. If the ssh connection failed in the first place, the tagged close will return an error tuple. In the example, this will be silent."},{"ref":"SSH.html#connect!/2","title":"SSH.connect!/2","type":"function","doc":"like connect/2 but raises with a ConnectionError instead of emitting an error tuple."},{"ref":"SSH.html#fetch/3","title":"SSH.fetch/3","type":"function","doc":"retrieves a binary file from the remote host. Under the hood, this uses the scp protocol to transfer files. The SCP protocol is as follows: execute scp remotely in the undocumented -f &lt;source&gt; mode send a single zero byte to initiate the conversation wait for a control string &quot;C0&lt;perms&gt; &lt;size&gt; &lt;filename&gt;&quot; send a single zero byte wait for the binary data + terminating zero send a single zero byte The perms term should be in octal, and the filename should be rootless. Example: SSH.fetch(conn, &quot;path/to/desired/file&quot;)"},{"ref":"SSH.html#fetch!/3","title":"SSH.fetch!/3","type":"function","doc":"like fetch/3 except raises instead of emitting an error tuple."},{"ref":"SSH.html#run/3","title":"SSH.run/3","type":"function","doc":"runs a command on the remote host. Options {iostream, redirect}: iostream may be either :stdout or :stderr. redirect may be one of the following: :stream sends the data to the stream. :stdout sends the data to the group_leader stdout. :stderr sends the data to the standard error io stream. :silent deletes all of the data. :raw sends the data to the stream tagged with source information as either {:stdout, data} or {:stderr, data}, as appropriate. {:file, path} sends the data to a new or existing file at the provided path. fun/1 processes the data via the function, with the output flat-mapped into the stream. this means that the results of fun/1 should be lists, with an empty list sending nothing into the stream. fun/2 is like fun/1 except the stream struct is passed as the second parameter. The output of fun/2 should take the shape {flat_map_results, modified_stream}. You may use the :data field of the stream struct to store arbitrary data; and a value of nil indicates that it has been unused. {:dir, path}: changes directory to path and then runs the command {:as, :binary} (default): outputs result as a binary {:as, :iolist}: outputs result as an iolist {:as, :tuple}: result takes the shape of the tuple {stdout_binary, stderr_binary} note that this mode will override any other redirection selected. Example: SSH.run(conn, &quot;hostname&quot;) # ==&gt; {:ok, &quot;hostname_of_remote\\n&quot;, 0} SSH.run(conn, &quot;some_program&quot;, stderr: :silent) # ==&gt; similar to running &quot;some_program 2&gt;/dev/null&quot; SSH.run(conn, &quot;some_program&quot;, stderr: :stream) # ==&gt; similar to running &quot;some_program 2&gt;&amp;1&quot; SSH.run(conn, &quot;some_program&quot;, stdout: :silent, stderr: :stream) # ==&gt; only capture standard error"},{"ref":"SSH.html#run!/3","title":"SSH.run!/3","type":"function","doc":"like run/3 except raises on errors instead of returning an error tuple. Note that by default this raises in the case that the SSH connection fails AND in the case that the remote command returns non-zero."},{"ref":"SSH.html#send/4","title":"SSH.send/4","type":"function","doc":"sends binary content to the remote host. Under the hood, this uses the scp protocol to transfer files. Protocol is as follows: execute scp remotely in the undocumented -t &lt;destination&gt; mode send a control string &quot;C0&lt;perms&gt; &lt;size&gt; &lt;filename&gt;&quot; wait for single zero byte send the binary data + terminating zero wait for single zero byte send EOF The perms term should be in octal, and the filename should be rootless. options: :permissions - sets unix-style permissions on the file. Defaults to 0o644 Example: SSH.send(conn, &quot;foo&quot;, &quot;path/to/desired/file&quot;)"},{"ref":"SSH.html#send!/4","title":"SSH.send!/4","type":"function","doc":"like send/4, except raises on errors, instead of returning an error tuple."},{"ref":"SSH.html#stream/3","title":"SSH.stream/3","type":"function","doc":"creates an SSH stream struct as an ok tuple or error tuple. Options {iostream, redirect}: iostream may be either :stdout or :stderr. redirect may be one of the following: :stream sends the data to the stream. :stdout sends the data to the group_leader stdout. :stderr sends the data to the standard error io stream. :silent deletes all of the data. :raw sends the data to the stream tagged with source information as either {:stdout, data} or {:stderr, data}, as appropriate. {:file, path} sends the data to a new or existing file at the provided path. fun/1 processes the data via the function, with the output flat-mapped into the stream. this means that the results of fun/1 should be lists, with an empty list sending nothing into the stream. fun/2 is like fun/1 except the stream struct is passed as the second parameter. The output of fun/2 should take the shape {flat_map_results, modified_stream}. You may use the :data field of the stream struct to store arbitrary data; and a value of nil indicates that it has been unused. {:stream_control_messages, boolean}: should the stream control messages :eof, or {:retval, integer} be sent to the stream? module: {mod, init}, The stream is operated using an module with behaviour SSH.ModuleApi data_timeout: timeout, how long to wait between packets till we send a timeout event."},{"ref":"SSH.html#stream!/3","title":"SSH.stream!/3","type":"function","doc":"like stream/2, except raises on an error instead of an error tuple."},{"ref":"SSH.html#t:chan/0","title":"SSH.chan/0","type":"type","doc":"channel reference for the SSH and SCP operations"},{"ref":"SSH.html#t:conn/0","title":"SSH.conn/0","type":"type","doc":"connection reference for the SSH and SCP operations"},{"ref":"SSH.html#t:ip4/0","title":"SSH.ip4/0","type":"type","doc":"erlang ip4 format, {byte, byte, byte, byte}"},{"ref":"SSH.html#t:remote/0","title":"SSH.remote/0","type":"type","doc":"connect to a remote is specified using either a domain name or an ip address"},{"ref":"SSH.html#t:retval/0","title":"SSH.retval/0","type":"type","doc":"unix-style return codes for ssh-executed functions"},{"ref":"SSH.Api.html","title":"SSH.Api","type":"behaviour","doc":"You might want to run mocked tests against a component that uses the SSH Api. Here is provided the behaviour module to support this testing pattern. generally speaking, these sorts of features should not be using the quick-and-dirty banged commands, so only the api for the fully error-tupled forms is provided."},{"ref":"SSH.Api.html#c:close/1","title":"SSH.Api.close/1","type":"callback","doc":""},{"ref":"SSH.Api.html#c:connect/1","title":"SSH.Api.connect/1","type":"callback","doc":""},{"ref":"SSH.Api.html#c:connect/2","title":"SSH.Api.connect/2","type":"callback","doc":""},{"ref":"SSH.Api.html#c:connect!/1","title":"SSH.Api.connect!/1","type":"callback","doc":""},{"ref":"SSH.Api.html#c:connect!/2","title":"SSH.Api.connect!/2","type":"callback","doc":""},{"ref":"SSH.Api.html#c:fetch/2","title":"SSH.Api.fetch/2","type":"callback","doc":""},{"ref":"SSH.Api.html#c:fetch/3","title":"SSH.Api.fetch/3","type":"callback","doc":""},{"ref":"SSH.Api.html#c:fetch!/2","title":"SSH.Api.fetch!/2","type":"callback","doc":""},{"ref":"SSH.Api.html#c:fetch!/3","title":"SSH.Api.fetch!/3","type":"callback","doc":""},{"ref":"SSH.Api.html#c:run/2","title":"SSH.Api.run/2","type":"callback","doc":""},{"ref":"SSH.Api.html#c:run/3","title":"SSH.Api.run/3","type":"callback","doc":""},{"ref":"SSH.Api.html#c:run!/2","title":"SSH.Api.run!/2","type":"callback","doc":""},{"ref":"SSH.Api.html#c:run!/3","title":"SSH.Api.run!/3","type":"callback","doc":""},{"ref":"SSH.Api.html#c:send/3","title":"SSH.Api.send/3","type":"callback","doc":""},{"ref":"SSH.Api.html#c:send/4","title":"SSH.Api.send/4","type":"callback","doc":""},{"ref":"SSH.Api.html#c:send!/3","title":"SSH.Api.send!/3","type":"callback","doc":""},{"ref":"SSH.Api.html#c:send!/4","title":"SSH.Api.send!/4","type":"callback","doc":""},{"ref":"SSH.ClientIdentity.html","title":"SSH.ClientIdentity","type":"module","doc":"implements erlang&#39;s :ssh_client_key_api behaviour so that you can log in using a pem key, instead of the default id_rsa private/public pair. It&#39;s pretty hard to believe that this is not an option, but there you go."},{"ref":"SSH.Config.html","title":"SSH.Config","type":"module","doc":"handles parsing of SSH configuration files. obtains configuration as follows: check the global config file, set by Application.put_env(:librarian, :global_config_file, &lt;file&gt;), or defaults to /etc/ssh/ssh_config. Alternatively may be set as an option in the load/1 function. You may suppress this by setting either the application environment variable or the option to false. This path must be an absolute path. check the user config file, set by Application.put_env(:librarian, :user_config_file, &lt;file&gt;), or defaults to ~/.ssh/config. Alternatively, may be set as an option in the load/1 function. You may suppress this by setting the application environment variable to false. This path may be a relative path. warnings Not all of the SSH options can be honored, only the ones that are supported by the erlang :ssh module. Moreover, not all of them have been implemented yet. If you need one, consider submitting a PR or an issue in the issue tracker. If your config file contains an alias for a certain activity, such as SSH tunneling, the config parser will still honor this alias, but will not provide the extra features specified in the config file. This behavior might change in the future or may be accompanied by a warning. wildcard configuration is not supported. argument tokens are not supported. Due to these warnings, using configuration is defaulted to false in SSH.connect/2. This behaviour may change in the future."},{"ref":"SSH.Config.html#load/1","title":"SSH.Config.load/1","type":"function","doc":"loads SSH configuration text from global and user configuration files. Defaults to checking /etc/ssh/ssh_config and ~/.ssh/config. options :global_config_path sets the global config path. Must be absolute. if set to false, will suppress checking global configs. :user_config_path sets the user config path. If set to false, will suppress checking local configs."},{"ref":"SSH.Config.html#parse/1","title":"SSH.Config.parse/1","type":"function","doc":"parses a configuration text, loaded as a string."},{"ref":"SSH.ConnectionError.html","title":"SSH.ConnectionError","type":"exception","doc":""},{"ref":"SSH.ModuleApi.html","title":"SSH.ModuleApi","type":"behaviour","doc":"API for rolling all of the SSH stream functions into a single module. This allows you to organize your stream IO transactions in a consistent fashion. Implementing this behaviour is particularly useful for interactive data streams - consider examining SSH.SCP.Send for a good example of how to implement a data stream that needs to exchange data in response to counterparty output."},{"ref":"SSH.ModuleApi.html#c:init/2","title":"SSH.ModuleApi.init/2","type":"callback","doc":"initializes your SSH stream. A return value of {:ok, stream} means that any preconditions for the stream have been successfully completed. You should return the initialized accumulator as the second part of the tuple. A return value of {:error, any} indicates an error in initialization, and the SSH channel will not be created."},{"ref":"SSH.ModuleApi.html#c:on_stderr/2","title":"SSH.ModuleApi.on_stderr/2","type":"callback","doc":"responds to binary data coming over the standard error segment of the data stream. See c:stdout/2, execpt with data coming over the standard error subchannel."},{"ref":"SSH.ModuleApi.html#c:on_stdout/2","title":"SSH.ModuleApi.on_stdout/2","type":"callback","doc":"responds to binary data coming over the standard out segment of the data stream. Note you may convert these data to multiple terms, which are then going to be marshalled as the stream output of the SSH stream as multiple terms. An empty list means that processing these data results in no effect. You may also emit :halt, which will stop the stream and cause the SSH client to send an EOF and close the stream. The output of this function is the modified stream. If you need to perform a reduction operation over your stream, the SSH.Stream struct provides an open-ended :data field which you may use to store your reducer. For interactive data streams, you may find it useful to call the SSH.Stream.send_data/2 or SSH.Stream.send_eof/1 functions within your implementations."},{"ref":"SSH.ModuleApi.html#c:on_timeout/1","title":"SSH.ModuleApi.on_timeout/1","type":"callback","doc":"responds to when it takes too long for the other side to respond with data. This is most useful for interactive data streams. By default, if you do not implement this function, the SSH.Stream module will pick an implementation that closes the connection."},{"ref":"SSH.RunError.html","title":"SSH.RunError","type":"exception","doc":""},{"ref":"SSH.SCP.Error.html","title":"SSH.SCP.Error","type":"exception","doc":""},{"ref":"SSH.SCP.FatalError.html","title":"SSH.SCP.FatalError","type":"exception","doc":""},{"ref":"SSH.SCP.Send.html","title":"SSH.SCP.Send","type":"module","doc":"implements the data transactions involved in a SCP file transfer to the destination server. Concretely, the following functions are implemented: init/2, for SSH.ModuleApi.init/2 on_stdout/2, for SSH.ModuleApi.on_stdout/2 on_stderr/2, for SSH.ModuleApi.on_stderr/2"},{"ref":"SSH.SCP.Send.html#init/2","title":"SSH.SCP.Send.init/2","type":"function","doc":"initializes the sending procedure for SCP. Expects the following information: the initialized SSH Stream. This will be modified to include our payload. a triple of: {remote_filepath, data_content, permissions} Permissions should be standard unix permissions scheme, for example 0o644 gives read and write permissions for the owner, and read permissions for everyone else, or 0o400 makes it read-only and only readable to the owner. The following tasks are performed: As per the SCP protocol, send the control command: C0&lt;permissions&gt; &lt;size&gt; &lt;remote_filepath&gt;. stash the content in the :data field of the stream."},{"ref":"SSH.SCP.Send.html#on_stderr/2","title":"SSH.SCP.Send.on_stderr/2","type":"function","doc":"responds to information returned on the stderr channel by the scp -t command. These strings will be saved to the stream as part of an {:stderr, data} tuple."},{"ref":"SSH.SCP.Send.html#on_stdout/2","title":"SSH.SCP.Send.on_stdout/2","type":"function","doc":"responds to information returned on the stdout channel by the scp -t command. As per the protocol the following binary responses can be expected: &lt;&lt;0&gt;&gt;: connection acknowledged, send data. &lt;&lt;0&gt;&gt;: data packet receieved, send more data. &lt;&lt;1, error_string&gt;&gt;: some error has occurred. &lt;&lt;2, error_string&gt;&gt;: some fatal error has occurred. The stream struct keeps track of what data it has sent via the :data field. If this contains data, then that data should be sent. If the the field contains :finished, then the available content has been exhausted. In general, no data will be emitted by the stream, although nonfatal errors will send an error tuple."},{"ref":"SSH.SCP.Send.html#on_timeout/1","title":"SSH.SCP.Send.on_timeout/1","type":"function","doc":"handles the host scp not calling back sometimes, the other side will forget to send a packet. In those cases, send a &lt;&lt;0&gt;&gt; packet down the SSH as reminder that we&#39;re still here and need a response."},{"ref":"SSH.SCP.Send.html#reducer/2","title":"SSH.SCP.Send.reducer/2","type":"function","doc":"postprocesses results sent to the ssh stream. The ssh stream is mostly used to return error values. We reduce on that stream as a final step in SSH.send/4. The two tuples we emit during stream processing are {:stderr, string} and {:error, string}. For an {:error, string} value, make that the reduced return value. For {:stderr, string}, send it to the process&#39;s standard error as a side effect."},{"ref":"SSH.Stream.html","title":"SSH.Stream","type":"module","doc":"Defines an SSH.Stream struct returned by SSH.stream!/3, as well as key functions that are involved in accessing the ssh data from the stream struct. Like IO.Stream, an SSH stream has side effects. Any given time you use it, the contents returned are likely to be different."},{"ref":"SSH.Stream.html#send_data/2","title":"SSH.Stream.send_data/2","type":"function","doc":"sends an iodata payload to the stdin of the ssh stream You should use this method inside of stderr, stdout, and data_timeout functions when you&#39;re designing interactive ssh handlers. Note that this function must be called from within the same process that the stream is running on, while the stream is running. In the future, we might write a guard that will prevent you from doing this from another process."},{"ref":"SSH.Stream.html#send_eof/1","title":"SSH.Stream.send_eof/1","type":"function","doc":"sends an end-of-file to the the ssh stream. You should use this method inside of stderr, stdout, and data_timeout functions when you&#39;re designing interactive ssh handlers. Note that this function must be called from within the same process that the stream is running on, while the stream is running. In the future, we might write a guard that will prevent you from doing this from another process."},{"ref":"SSH.Stream.html#t:control_message/0","title":"SSH.Stream.control_message/0","type":"type","doc":"ssh protocol stream control messages"},{"ref":"SSH.Stream.html#t:iostream_message/0","title":"SSH.Stream.iostream_message/0","type":"type","doc":"binary data sent over the server&#39;s standard out (0) or standard error (1)"},{"ref":"SSH.Stream.html#t:outbound_message/0","title":"SSH.Stream.outbound_message/0","type":"type","doc":"messages that the local client can use to send streaming content"},{"ref":"SSH.Stream.html#t:process_fn/0","title":"SSH.Stream.process_fn/0","type":"type","doc":"a lambda that acts on channel data and convert it to stream tokens"},{"ref":"SSH.Stream.html#t:ssh_message/0","title":"SSH.Stream.ssh_message/0","type":"type","doc":"all messages that are blocked by the stream control loop. Note that most of these are the third term in a {:ssh_cm, conn, &lt;message&gt;} tuple."},{"ref":"SSH.Stream.html#t:stream_control_tokens/0","title":"SSH.Stream.stream_control_tokens/0","type":"type","doc":"the default tokens that can be sent for stream processing"},{"ref":"SSH.Stream.html#t:stream_tokens/0","title":"SSH.Stream.stream_tokens/0","type":"type","doc":"the default tokens that can be sent for stream processing"},{"ref":"SSH.Stream.html#t:t/0","title":"SSH.Stream.t/0","type":"type","doc":"the stream data structure"},{"ref":"SSH.StreamError.html","title":"SSH.StreamError","type":"exception","doc":""},{"ref":"readme.html","title":"Librarian","type":"extras","doc":"Librarian An Elixir SSH library Elixir&#39;s SSH offering needs a major revamp. :librarian provides a module, SSH, that is well-typed, and, under the hood, uses Elixir&#39;s Stream module. Librarian does not create any worker pools or have any opinions on how you should manage processes or supervision trees. Future releases may provide batteries-included solutions for these concerns, but Librarian will never instantiate running processes in your BEAM without an explicit command to do so.Currently, because librarian uses Stream it cannot multiplex ssh channels on a single BEAM process, but support for those uses cases may be forthcoming."},{"ref":"readme.html#usage","title":"Librarian - Usage","type":"extras","doc":"Warning the app :librarian defines the SSH module. We can&#39;t call the package :ssh due to conflicts with the erlang :ssh builtin module."},{"ref":"readme.html#supported-platforms","title":"Librarian - Supported Platforms","type":"extras","doc":":librarian is currently only tested on linux. MacOSX should in theory work, and there are parts that will probably not work on Windows. Any assistance getting these platforms up and running would be appreciated. For Mix Tasks and releases If you would like to use Librarian in Mix Tasks or Releases, you should make sure that Application.ensure_all_running(:ssh) has been called before you attempt any Librarian commands, or else you may wind up with a race condition, since OTP may take a while to get its default :ssh package up and running."},{"ref":"readme.html#examples","title":"Librarian - Examples","type":"extras","doc":"NB all of these commands assume that you have passwordless ssh keys to the server &quot;some.other.server&quot;, to the user with the same username as the currently running BEAM VM. For help with other uses, consult the documentation. {:ok, conn} = SSH.connect(&quot;some.other.server&quot;) SSH.run!(conn, &quot;echo hello ssh&quot;) # ==&gt; &quot;hello ssh&quot; Librarian provides two scp-related commands, fetch and send, which let you fetch or send individual files. remote_file_binary_content = &quot;some.other.server&quot; |&gt; SSH.connect! |&gt; SSH.fetch!(&quot;remote_file.txt&quot;) &quot;some.other.server&quot; |&gt; SSH.connect! |&gt; SSH.send!(&quot;foo bar&quot;, &quot;remote_foo_bar.txt&quot;) For all three of these operations, Librarian provides an ok tuple form or a bang form. See the documentation for the details on these forms. Finally, you can use the underlying SSH stream functionality directly, by emitting a stream of values: &quot;some.other.server&quot; |&gt; SSH.connect! |&gt; SSH.stream!(&quot;some_long_running_process&quot;) |&gt; Enum.each(SomeModule.some_action/1) Like the IO.Stream struct, the SSH.Stream struct emitted by SSH.stream!/2 is both an Enumerable and a Collectable, so you can use it to accept a datastream to send to the standard in of your remote ssh command. conn = SSH.connect!(&quot;some.other.server&quot;) 1..1000 |&gt; Stream.map(&amp;(inspect(&amp;1) &lt;&gt; &quot;\\n&quot;)) |&gt; Enum.into(SSH.stream!(&quot;tee &gt; one_to_one_thousand.txt&quot;)) These are the basic use cases. You can find more information about more advanced use cases at https://hexdocs.pm/librarian."},{"ref":"readme.html#installation","title":"Librarian - Installation","type":"extras","doc":"This package can be installed by adding librarian to your list of dependencies in mix.exs: def deps do [ {:librarian, &quot;~&gt; 0.1.0&quot;} ] end documentation can be found here: https://hexdocs.pm/librarian/SSH.html"},{"ref":"readme.html#testing","title":"Librarian - Testing","type":"extras","doc":"Almost all of Librarian&#39;s tests are integration tests. These tests are end-to-end and run against an active, default Linux SSH server. To properly run these tests, you should have the following: The latest version of OpenSSH A default, passwordless id_rsa.pub in your ~/.ssh directory. [ -f ~/.ssh/id_rsa.pub ] || ssh-keygen -t rsa -N &quot;&quot; -f ~/.ssh/id_rsa.pub The default key as an authorized key. cat ~/.ssh/id_rsa.pub &gt;&gt; ~/.ssh/authorized_keys We will be working on setting up alternative testing strategies in the future."}]